package ProjectElectricLocomotive.src.Drawnings;


import ProjectElectricLocomotive.src.DirectionType;

import ProjectElectricLocomotive.src.Entities.EntityLocomotive;

import java.awt.*;
import java.util.Random;

public class DrawingLocomotive {
    public EntityLocomotive _entityLocomotive;
    public EntityLocomotive GetEntityLocomotive()
    {
        return _entityLocomotive;
    }
    protected Integer _startPosX = null;
    public Integer getPosX() {
        return _startPosX;
    }
    protected Integer _startPosY = null;
    public Integer getPosY() {
        return _startPosY;
    }
    private Integer _pictureWidth = null;
    public int getWidth() {
        return _drawingLocomotiveWidth;
    }
    private Integer _pictureHeight = null;
    public int getHeight() {
        return _drawingLocomotiveHeight;
    }
    private int _drawingLocomotiveWidth = 83;
    private int _drawingLocomotiveHeight = 45;

    public DrawingLocomotive(int speed, float weight, Color bodyColor)
    {
        this._entityLocomotive = new EntityLocomotive(speed, weight, bodyColor);
    }

    private DrawingLocomotive()
    {
        _pictureWidth = null;
        _pictureHeight = null;
        _startPosX = null;
        _startPosY = null;
    }
    protected DrawingLocomotive(int drawningLocomotiveWidth, int drawningLocomotiveHeight)
    {
        this._drawingLocomotiveWidth = drawningLocomotiveWidth;
        this._drawingLocomotiveHeight = drawningLocomotiveHeight;
    }

    public boolean SetPictureSize(int width, int height)
    {
        if (_drawingLocomotiveWidth > width || _drawingLocomotiveHeight > height)
        {
            return false;
        }
        _pictureWidth = width;
        _pictureHeight = height;
        if (_startPosX != null && _drawingLocomotiveWidth + _startPosX > width) _startPosX = _pictureWidth - _drawingLocomotiveWidth;
        if (_startPosY != null && _startPosY + _drawingLocomotiveHeight > _pictureHeight) _startPosY = _pictureHeight - _drawingLocomotiveHeight;
        return true;
    }
    public void SetPosition(int x, int y)
    {
        if (_pictureHeight == null || _pictureWidth == null) return;
        _startPosX = x;
        _startPosY = y;
        if (x + _drawingLocomotiveWidth > _pictureWidth || y + _drawingLocomotiveHeight > _pictureHeight || x < 0 || y < 0)
        {
            if (x < 0) _startPosX = 0;
            if (y < 0 ) _startPosY = 0;
            if (x + _drawingLocomotiveWidth > _pictureWidth) _startPosX = _pictureWidth - _drawingLocomotiveWidth;
            if (y + _drawingLocomotiveHeight > _pictureHeight) _startPosY = _pictureHeight - _drawingLocomotiveHeight;
        }
    }

    public boolean MoveTransport(DirectionType direction)
    {
        if( _startPosX == null || _startPosY == null) return false;
        switch (direction)
        {
            // Влево
            case Left:
                if(_startPosX - _entityLocomotive.Step >= 0)
                {
                    _startPosX -= (int)_entityLocomotive.Step;
                }
                return true;
            // Вправо
            case Right:
                if(_startPosX + _entityLocomotive.Step + _drawingLocomotiveWidth < _pictureWidth)
                {
                    _startPosX += (int)_entityLocomotive.Step;
                }
                return true;
            // Вверх
            case Up:
                if(_startPosY - _entityLocomotive.Step >= 0)
                {
                    _startPosY -= (int)_entityLocomotive.Step;
                }
                return true;
            // Вниз
            case Down:
                if(_startPosY + _entityLocomotive.Step + _drawingLocomotiveHeight < _pictureHeight)
                {
                    _startPosY += (int)_entityLocomotive.Step;
                }
                return true;
            default:
                return false;
        }
    }
    public void DrawTransport(Graphics g) {
        if (_entityLocomotive == null || _startPosX == null || _startPosY == null) return;

        // Создание перьев и кистей для прорисовки электровоза
        Color blackColor = Color.BLACK;
        Color deepSkyBlueColor = new Color(0, 191, 255);
        Color yellowColor = Color.YELLOW;

        // Инициализация опорных точек для прорисовки корпуса
        int pointStartX = _startPosX + 75;
        int pointStartY = _startPosY + 20;

        Point pointStart = new Point(pointStartX, pointStartY);
        Point point1 = new Point(pointStartX, pointStartY - 15);
        Point point2 = new Point(_startPosX + 10, pointStartY - 15);
        Point pointFinish = new Point(_startPosX + 5, pointStartY);

        // Инициализация опорных точек для прорисовки первой "юбки"
        Point point3 = new Point(_startPosX + 7, _startPosY + 37);
        Point point4 = new Point(_startPosX, _startPosY + 43);
        Point point5 = new Point(_startPosX + 7, _startPosY + 43);

        // Инициализация опорных точек для прорисовки второй "юбки"
        Point point6 = new Point(_startPosX + 69, _startPosY + 37);
        Point point7 = new Point(_startPosX + 82, _startPosY + 43);
        Point point8 = new Point(_startPosX + 69, _startPosY + 43);

        // Совокупность точек полигона уголка первой "юбки"
        Point[] firstTrianglePoints = { point3, point4, point5 };

        // Совокупность точек полигона уголка второй "юбки"
        Point[] secondTrianglePoints = { point6, point7, point8 };

        g.setColor(blackColor);
        g.fillOval(_startPosX + 58, _startPosY + 37, 10, 10);
        g.drawOval(_startPosX + 58, _startPosY + 37, 8, 8);
        g.fillOval(_startPosX + 48, _startPosY + 37, 10, 10);
        g.drawOval(_startPosX + 48, _startPosY + 37, 8, 8);
        g.fillOval(_startPosX + 19, _startPosY + 37, 10, 10);
        g.drawOval(_startPosX + 19, _startPosY + 37, 8, 8);
        g.fillOval(_startPosX + 9, _startPosY + 37, 10, 10);
        g.drawOval(_startPosX + 9, _startPosY + 37, 8, 8);

        // Прорисовка уголков "юбок"
        g.setColor(_entityLocomotive.GetBodyColor());
        g.fillPolygon(new Polygon(new int[]{ point3.x, point4.x, point5.x }, new int[]{ point3.y, point4.y, point5.y }, 3));
        g.fillPolygon(new Polygon(new int[]{ point6.x, point7.x, point8.x }, new int[]{ point6.y, point7.y, point8.y }, 3));

        // Прорисовка корпуса
        g.drawRect(_startPosX + 5, _startPosY + 20, 70, 17);
        g.drawLine(point3.x, point3.y, point4.x, point4.y);
        g.drawLine(pointStart.x, pointStart.y, point1.x, point1.y);
        g.drawLine(point1.x, point1.y, point2.x, point2.y);
        g.drawLine(point2.x, point2.y, pointFinish.x, pointFinish.y);

        // Прорисовка окон
        g.setColor(deepSkyBlueColor);
        g.drawRect(_startPosX + 12, _startPosY + 9, 8, 8);
        g.drawRect(_startPosX+ 63, _startPosY + 9, 8, 8);
    }
}
